import os.path

import numpy as np
from einops import rearrange
import rasterio
from rasterio.transform import from_origin
import netCDF4 as nc


def save_np2tif(np_data, output_file, coordinate=None, resolution=None):
    h, w = np_data.shape
    if coordinate:
        # 创建带地理坐标信息的 GeoTIFF 文件
        left, top = coordinate
        res1, res2 = resolution
        # 左上角坐标， 先经后纬
        transform = from_origin(left, top, res1, res2)
        with rasterio.open(output_file, "w", driver="GTiff", width=w, height=h, count=1, dtype=np_data.dtype,
                           crs="EPSG:4326", transform=transform) as dst:
            dst.write(np_data, 1)
        print(f"GeoTIFF '{output_file}' generated with geographic coordinates.")
    else:
        # 将数据写入TIFF文件
        with rasterio.open(output_file, "w", width=w, height=h, count=1, dtype=np_data.dtype) as dst:
            dst.write(np_data, 1)
        print(f"TIFF image saved as '{output_file}'")


def np2tif(input_data, save_path, left=None, top=None, x_res=None, y_res=None):
    if type(input_data) == str:
        np_data = np.load(input_data)
    else:
        np_data = input_data
    shape = np_data.shape
    if len(shape) < 2:
        raise Exception(str(shape) + 'is less than 2 Dimensions')
    mode_list = ['d' + str(i) for i in range(len(shape))]

    mode_str = ' '.join(mode_list) + '->(' + ' '.join(mode_list[:-2]) + ') ' + mode_list[-2] + ' ' + mode_list[-1]
    np_data = rearrange(np_data, mode_str)
    if left is not None and top is not None:
        coordinate = (left, top)
    else:
        coordinate = None
    for idx, single_np in enumerate(np_data):
        name = ''
        idx_tmp = idx
        for s in range(len(shape[:-2])):
            temp = idx_tmp // np.prod(shape[s + 1:-2], axis=None)
            name += str(int(temp)) + '-'
            idx_tmp -= temp * np.prod(shape[s + 1:-2], axis=None)
        save_np2tif(single_np, os.path.join(save_path, name[:-1] + '.tif'), coordinate=coordinate,
                    resolution=(x_res, y_res))


def nc2np(input_data):
    if type(input_data) == str:
        nc_data = nc.Dataset(input_data)  # 读取.nc文件，传入f中。此时f包含了该.nc文件的全部信息
    else:
        nc_data = input_data
    vars = []
    max_shape = 0
    for name, var in nc_data.variables.items():
        if len(var.shape) > max_shape:
            max_shape = len(var.shape)
            vars = [name]
        elif len(var.shape) == max_shape:
            vars.append(name)
    np_data = []
    for var in vars:
        np_data.append(np.array(nc_data[var][:]))
    np_data = np.array(np_data)
    return np_data


if __name__ == '__main__':
    nc_t = nc2np(r'C:\Users\ECNU\Desktop\upper.nc')
    np2tif(nc_t, 'files')
    np2tif(nc_t, 'files', left=0, top=90, x_res=0.25, y_res=0.25)
